package com.maximussoft.socketssl;

import android.app.Application;
import android.util.Log;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.AbsObjectWrapper;

@ShortName("SocketSSL")
@Version(1.2F)
@Author("elbarto,thedesolatesoul")
@Permissions(values = { "android.permission.INTERNET",
		"android.permission.ACCESS_NETWORK_STATE" })

public class SocketSSL {

	private volatile SSLSocket socket;
	private String eventName;

	/**
	 * 
	 * Initialize SSL Socket.
	 */
	public void Initialize(String EventName, String Protocol) throws NoSuchAlgorithmException, KeyManagementException, IOException {
		Log.w("B4A", "** Start initialize SSL Socket **");
		SSLContext sslcontext = SSLContext.getInstance(Protocol);
		Log.w("B4A", "** Protocol used: " + sslcontext.getProtocol());
		sslcontext.init(null, trustAllCerts, new SecureRandom());
		SSLSocketFactory socketFactory = sslcontext.getSocketFactory();
		this.socket = (SSLSocket) socketFactory.createSocket();
		this.eventName = EventName.toLowerCase(BA.cul);
		Log.w("B4A", "** Emd initialize SSL Socket **");
	}
	
	public static void LIBRARY_DOC() {
	}

	/**
	 * 
	 * Returns if SSL Socket is initialized or not.
	 */
	public boolean IsInitialized() {
		return this.socket != null;
	}

	/**
	 * 
	 * Returns the raw IP address in a string format.
	 */
	public String ResolveHost(String Host) throws UnknownHostException {
		return InetAddress.getByName(Host).getHostAddress();
	}
	
	/**
	 * 
	 * Gets or sets the timeout.
	 */
	public int getTimeOut() throws SocketException {
		return this.socket.getSoTimeout();
	}
	/**
	 * 
	 * Gets or sets the timeout.
	 */
	public void setTimeOut(int value) throws SocketException {
		this.socket.setSoTimeout(value);
	}
	
	/**
	 * 
	 * Connect SSL Socket.
	 */
	public void Connect(final BA ba, final String Host, final int Port,
			final int TimeOut) throws UnknownHostException {
		Runnable r = new Runnable() {

			@Override
			public void run() {
				Log.w("B4A", "** Start connect SSL Socket **");
				SSLSocket mySocket = socket;
				try {
					InetAddress Address = InetAddress.getByName(Host);
					InetSocketAddress i = new InetSocketAddress(Address, Port);
					if (mySocket != socket)
						return;
					socket.setUseClientMode(true);
					socket.connect(i, TimeOut);
					socket.startHandshake();
					Log.w("B4A", "** SSL Socket Event OK **");
					ba.raiseEventFromDifferentThread(SocketSSL.this, null,
							0, eventName + "_connected", true,
							new Object[] { true });
				} catch (Exception e) {
					Log.w("B4A", "", e);
					if (mySocket == socket) {
						ba.setLastException(e);
						Log.w("B4A", "** SSL Socket Event Error **");
						ba.raiseEventFromDifferentThread(SocketSSL.this,
								null, 0, eventName + "_connected", true,
								new Object[] { false });
					}
				}
				Log.w("B4A", "** End connect SSL Socket **");
			}

		};
		ba.submitRunnable(r, this, 0);
	}
	
	/**
	 * 
	 * SSL Socket InputStream.
	 */
	public InputStream getInputStream() throws IOException {
		return this.socket.getInputStream();
	}

	/**
	 * 
	 * SSL Socket OutputStream.
	 */
	public OutputStream getOutputStream() throws IOException {
		return this.socket.getOutputStream();
	}

	/**
	 * 
	 * SSL Socket connected or not.
	 */
	public boolean getConnected() {
		return (this.socket != null) && (this.socket.isConnected());
	}

	/**
	 * 
	 * Close SSL Socket.
	 */
	public void Close() throws IOException {
		if (this.socket != null) {
			SSLSocket s = this.socket;
			this.socket = null;
			s.close();
		}
	}

	TrustManager[] trustAllCerts = new TrustManager[] { new X509TrustManager() {

		public java.security.cert.X509Certificate[] getAcceptedIssuers() {
			return new java.security.cert.X509Certificate[] {};
		}

		@Override
		public void checkClientTrusted(
				java.security.cert.X509Certificate[] chain, String authType)
				throws java.security.cert.CertificateException {
		}

		@Override
		public void checkServerTrusted(
				java.security.cert.X509Certificate[] chain, String authType)
				throws java.security.cert.CertificateException {
		}
	} };

}
